package com.gmei.data.dqmp.pool;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.LinkedList;
import java.util.Properties;

public class JdbcConnectPoolBak02 {

	private static final Logger logger = LoggerFactory.getLogger(JdbcConnectPoolBak02.class);
	private static Properties prop;
	private static int requestCount = 0;
	private static LinkedList<Connection> connList = new LinkedList<Connection>();

	static {
		prop = getProperties("jdbc.properties");
		try {
			Class.forName(prop.getProperty("driverClassName"));
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}

	public static Connection getConnection() {
		Connection connection = null;
		if (connList.size() > 0) {
			connection = connList.getFirst();
			connList.removeFirst();
		} else{
			connList.addLast(createConnect());
			connection = getConnection();
		}
		logger.info("requestCount : {}",++requestCount);
		logger.info("connList size : {}",connList.size());
		return connection;
	}

	/**
	 * 创建连接
	 * 
	 * @return
	 */
	private static Connection createConnect() {
		Connection conn = null;
		try {
			conn = DriverManager.getConnection(
					prop.getProperty("url"),
					prop.getProperty("username"),
					prop.getProperty("password"));
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return conn;
	}

	/**
	 * 释放连接
	 * 
	 * @param connection
	 */
	public static void releaseConnection(Connection connection) {
		connList.addLast(connection);
	}

	/**
	 * 获取属性文件对象
	 * 
	 * @param path
	 * @return
	 */
	private static Properties getProperties(String path) {
		Properties properties = new Properties();
		try {
			InputStream inputStream = JdbcConnectPoolBak02.class.getClassLoader().getResourceAsStream(path);
			properties.load(inputStream);
			inputStream.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return properties;
	}
}
