package com.gmei.data.dqmp.service.impl;

import java.util.Date;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gmei.data.dqmp.common.Constants;
import com.gmei.data.dqmp.domain.TblMonitorVolatilityRule;
import com.gmei.data.dqmp.domain.TblMonitorVolatilityRuleCriteria;
import com.gmei.data.dqmp.domain.TblResultMonitorVolatility;
import com.gmei.data.dqmp.domain.TblResultMonitorVolatilityCriteria;
import com.gmei.data.dqmp.domain.TblResultMonitorVolatilityCriteria.Criteria;
import com.gmei.data.dqmp.mapper.TblMonitorVolatilityRuleMapper;
import com.gmei.data.dqmp.mapper.TblResultMonitorVolatilityMapper;
import com.gmei.data.dqmp.service.DingdingService;
import com.gmei.data.dqmp.service.MailService;
import com.gmei.data.dqmp.service.WarningService;
import com.gmei.data.dqmp.utils.CrontabUtils;
import com.gmei.data.dqmp.utils.DateUtils;

@Service
public class WarningServiceImpl implements WarningService {

	@Autowired
	private DingdingService dingdingService;
	@Autowired
	private MailService mailService;
	@Autowired
	private TblMonitorVolatilityRuleMapper tblMonitorVolatilityRuleMapper;
	@Autowired
	private TblResultMonitorVolatilityMapper tblResultMonitorVolatilityMapper;
	private static final Logger logger = LoggerFactory.getLogger(WarningServiceImpl.class);

	@Override
	public void checkAndWarning() {
		List<TblMonitorVolatilityRule> tblMonitorVolatilityRuleList = tblMonitorVolatilityRuleMapper
				.selectByExample(new TblMonitorVolatilityRuleCriteria());
		Date currentDate = new Date();
		String lastOneDayDateStr = DateUtils.getLastNumDaysDateStr(1);
		String lastTwoDayDateStr = DateUtils.getLastNumDaysDateStr(2);
		for (TblMonitorVolatilityRule tblMonitorVolatilityRule : tblMonitorVolatilityRuleList) {
			if (Constants.MONITOR_TYPE.equals(tblMonitorVolatilityRule.getMonitorType())) {
				String checkTime = tblMonitorVolatilityRule.getCheckTime();
				String dbName = tblMonitorVolatilityRule.getDbName();
				String tbName = tblMonitorVolatilityRule.getTbName();
				if (!CrontabUtils.isToRunByMinites(currentDate, checkTime)) {
					continue;
				}
				TblResultMonitorVolatilityCriteria tblResultMonitorVolatilityCriteria = new TblResultMonitorVolatilityCriteria();
				Criteria criteria = tblResultMonitorVolatilityCriteria.createCriteria();
				criteria.andDbNameEqualTo(dbName).andTbNameEqualTo(tbName)
						.andColNameEqualTo(tblMonitorVolatilityRule.getColName())
						.andIndicatorTypeEqualTo(tblMonitorVolatilityRule.getIndicatorType())
						.andPartitionDateBetween(lastTwoDayDateStr, lastOneDayDateStr);
				tblResultMonitorVolatilityCriteria.setOrderByClause("partition_date");
				List<TblResultMonitorVolatility> tblResultMonitorVolatilityList = tblResultMonitorVolatilityMapper
						.selectByExample(tblResultMonitorVolatilityCriteria);
				Long lastOneDayIndicatorValue = null, lastTwoDayIndicatorValue = null;
				for (TblResultMonitorVolatility tblResultMonitorVolatility : tblResultMonitorVolatilityList) {
					String partitionDate = tblResultMonitorVolatility.getPartitionDate();
					if (lastOneDayDateStr.equals(partitionDate)) {
						lastOneDayIndicatorValue = tblResultMonitorVolatility.getIndicatorValue();
					} else if (lastTwoDayDateStr.equals(partitionDate)) {
						lastTwoDayIndicatorValue = tblResultMonitorVolatility.getIndicatorValue();
					} else {
						logger.error("PartitionDate value is error: {}", partitionDate);
						continue;
					}
				}
				if (lastOneDayIndicatorValue == null || lastTwoDayIndicatorValue == null) {
					logger.error("The lastOneDayIndicatorValue or lastTwoDayIndicatorValue or all values of table {} is empty!", tblMonitorVolatilityRule.getTbName());
					return;
				}
				Double rs = Math.abs((lastOneDayIndicatorValue - lastTwoDayIndicatorValue)
						/ Double.valueOf(lastTwoDayIndicatorValue));
				Double threshold = tblMonitorVolatilityRule.getThreshold();
				if (rs > threshold) {
					String warningMsg = String.format("表%s.%s中数据量变化超过%s，疑似异常，请核实!", dbName, tbName,threshold * 100 + "%");
					mailService.sendSimpleMail(warningMsg);
					dingdingService.sendMsgToDingding(warningMsg);
					logger.error("表{}.{}中数据量变化超过{}，疑似异常，请核实!", dbName, tbName,threshold * 100+"%");
				}
			}
		}
	}
}
